@@ -1,12 +1,12 @@
 /*****************************************************************************
- * Secure Locate v2.0                                                        *
- * July 1, 1999                                                              *
+ * Secure Locate v2.1                                                        *
+ * October 19, 1999                                                          *
  * Programmed by: Kevin Lindsay                                              *
  * Copyright (c) 1998 Kevin Lindsay & Netnation Communications Inc.          *
  *                                                                           *
  * NetNation Communications   http://www.netnation.com/                      *
  * Secure Locate: ftp://ftp.mkintraweb.com/pub/linux/slocate/                *
- *                http://pep.netnation.com/slocate/                          *
+ *                http://pep.stormix.com/slocate/                            *
  *                                                                           *
  * Patches:  Sean Mcnulty <lazy@ccf.rutgers.edu>                             *
  *                * Fixed a bug which caused the decode function to fail.    *
@@ -28,6 +28,12 @@
  *                  To Alex, slocate -u is much faster!                      *
  *           Matt Heckaman <matt@MLINK.NET>                                  *
  *                * Created a patch to make Secure Locate work with FreeBSD. *
+ *           Luca Berra <bluca@vodka.it>                                     *
+ *                * Added case insensitive option and optimized code to make *
+ *                  searching faster.                                        *
+ *           Hans-Juergen Godau <godau@wi-inf.uni-essen.de>                  *
+ *                * Fixed a segfault when searching through more than one    *
+ *                  database.                                                *
  *                                                                           *
  * Report any Bugs to: klindsay@mkintraweb.com                               *
  *                                                                           *
@@ -85,11 +91,12 @@
 # include <sys/param.h>
 # include <sys/ucred.h>
 # include <sys/mount.h>
+# undef FNM_CASEFOLD
 #endif
 
 /* GLOBALS */
 
-#define VERSION "Secure Locate v2.0 - Released July 1, 1999\n"
+#define VERSION "Secure Locate v2.1 - Released October 19, 1999\n"
 #define GRPFILE "/etc/group"
 #define SLOC_ESC 0x80
 #define SLOC_GRP "slocate"
@@ -122,6 +129,7 @@ char *EXCLUDE_DIR=NULL;
 int EXCLUDE=0;
 int VERBOSE=0;
 int QUIET=0;
+int NOCASE=0;
 int REGEXP=0;
 int NEWOUTPUT=0;
 char *regexp;
@@ -148,7 +156,7 @@ usage()
    int i;
    printf("%s\n"
           "Copyright (c) 1999 Kevin Lindsay & Netnation Communications Inc.\n\n"
-          "search usage:   %s [-q] [-d <path>] [--database=<path>] <search string>...\n"
+          "search usage:   %s [-qi] [-d <path>] [--database=<path>] <search string>...\n"
           "                %s [-r <regexp>] [--regexp=<regexp>]\n"
           "database usage: %s [-qv] [-o <file>] [--output=<file>]\n"
           "                %s [-e <dir1,dir2,...>] [-f <fs_type1,...> ] [-l <level>]\n"
@@ -161,7 +169,7 @@ usage()
 #ifndef __FreeBSD__
           "                  [-c] <[-U <path>] [-u]>\n"
 #else
-          "                  <[-U <path>] [-u]>\n"          
+          "                  <[-U <path>] [-u]>\n"
 #endif
           "general usage:  %s [-Vh] [--version] [--help]\n\n"
           "   Options:\n"
@@ -184,6 +192,7 @@ usage()
           "                           1 turns security checks on. This is the default.\n"
           "   -q                 - Quiet mode.  Error messages are suppressed.\n"
           "   -n <num>           - Limit the amount of results shown to <num>.\n"
+          "   -i                 - Does a case insensitive search.\n"
           "   -r <regexp>\n"
           "   --regexp=<regexp>  - Search the database using a basic POSIX regular\n"
           "                        expression.\n"
@@ -201,7 +210,7 @@ usage()
           "Author: Kevin Lindsay\n"
           "Bugs:   klindsay@mkintraweb.com\n"
           "FTP:    ftp://ftp.mkintraweb.com/pub/linux/slocate/\n"
-          "HTTP:   http://pep.netnation.com/slocate/\n"
+          "HTTP:   http://pep.stormix.com/slocate/\n"
           "\n");
    
    exit(0);
@@ -236,7 +245,7 @@ parse_create_path(char *path)
    if(ret_val==-1)
    {
       if (!QUIET)
-        printf("%s: could not create database: %s: %s\n", progname,path,strerror(errno));
+        fprintf(stderr,"%s: could not create database: %s: %s\n", progname,path,strerror(errno));
       exit(1);
    }
       
@@ -958,6 +967,13 @@ decode_db(char *database, char *str)
    regmatch_t pmatch[32];
    int cur_queries = 0;
    int reg_res;
+   int foundit = 0;
+#ifndef FNM_CASEFOLD
+   char *casestr=NULL;
+   char *casecodedpath=NULL;
+   char *cp=NULL;
+#endif
+     
    
    if ((fd = fopen(database,"r")) == NULL) {
       if (!QUIET)
@@ -976,7 +992,7 @@ decode_db(char *database, char *str)
    time(&now);
    if (now - statres.st_mtime > WARN_SECONDS)
    {
-      fprintf(stdout,"%s: warning: database %s' is more than %s old\n",progname,database,WARN_MESSAGE);
+      fprintf(stderr,"%s: warning: database %s' is more than %s old\n",progname,database,WARN_MESSAGE);
    }   
    
    slevel = getc(fd);
@@ -990,6 +1006,24 @@ decode_db(char *database, char *str)
        ((globptr1 = strchr(str,'[')) != NULL && strchr(str,']') != NULL))
        globflag = 1;
    
+   if (REGEXP) {   
+      preg = malloc(sizeof(regex_t));
+      if ((reg_res = regcomp(preg,regexp,NOCASE?REG_ICASE:0)) != 0) {
+         regerror(reg_res, preg, errbuf,1024);
+         if (!QUIET)
+           fprintf(stderr,"error: %s: regular expression: %s\n",progname,errbuf);
+         exit(1);
+      }
+#ifndef FNM_CASEFOLD
+      
+   } else if (NOCASE) {
+      casestr=strdup(str);
+      for (cp = casestr; *cp; cp++)
+        *cp = tolower(*cp);
+#endif /* FNM_CASEFOLD */
+      
+   }
+   
    while ((code_num = getc(fd)) != EOF) {
       
       if (code_num == SLOC_ESC) {
@@ -1034,18 +1068,36 @@ decode_db(char *database, char *str)
       printit=0;
       
       if (REGEXP) {
-         globflag=0;
-         preg = malloc(sizeof(regex_t));
-         if ((reg_res = regcomp(preg,regexp,0)) != 0) {
-            regerror(reg_res, preg, errbuf,1024);
-            if (!QUIET)
-              fprintf(stdout,"error: %s: regular expression: %s\n",progname,errbuf);
-            exit(1);
-         }         
-         
-      }
+         foundit = !regexec(preg,codedpath,nmatch,pmatch,0);         
+      } else if (NOCASE) {
+#ifdef FNM_CASEFOLD /* i suppose i also have strcasestr */
+         if (globflag) {
+            foundit=!fnmatch(str,codedpath,FNM_CASEFOLD);
+            
+         } else {
+            foundit=(strcasestr(codedpath,str) != NULL);
+         }
+#else /* FNM_CASEFOLD */
+         casecodedpath=strdup(codedpath);
+         for (cp = casecodedpath; *cp; cp++)
+           *cp = tolower(*cp);
+         if (globflag) {
+            foundit=!fnmatch(casestr,casecodedpath,0);
+         } else {
+            foundit=(strstr(casecodedpath,casestr) != NULL);            
+         }
+         free(casecodedpath);
+#endif /* FNM_CASEFOLD */
 
-      if ((globflag && !fnmatch(str,codedpath,0)) || (!REGEXP && strstr(codedpath,str) != NULL) || (REGEXP && regexec(preg,codedpath,nmatch,pmatch,0) == 0)) {
+      } else {
+         if (globflag) {
+            foundit=!fnmatch(str,codedpath,0);
+         } else {
+            foundit=(strstr(codedpath,str) != NULL);
+         }
+      }
+      
+      if (foundit) {
          if (slevel == '1') {
             if (UID == 0 || lstat(codedpath,&statres) == 0) {
                printit=1;
@@ -1064,9 +1116,6 @@ decode_db(char *database, char *str)
              printit=1;
       }
       
-      if (REGEXP)
-        regfree(preg);
-      
       if (printit) {
          res = 0;
          cur_queries += 1;
@@ -1074,7 +1123,15 @@ decode_db(char *database, char *str)
          if (max_queries > 0 && cur_queries == max_queries)
            exit(0);
       }
-   }      
+   }
+   
+   if (REGEXP)
+     regfree(preg);
+#ifndef FNM_CASEFOLD
+   else if (NOCASE)
+     free(casestr);
+#endif /* FNM_CASEFOLD */
+     
    
    fclose(fd);
    
@@ -1129,7 +1186,7 @@ main(int args, char **argv)
    parse_decode_path(SLOCATEDB);
    parse_decode_path(getenv("LOCATE_PATH"));
 
-   while ((ch = getopt(args,argv,"VvuhqU:r:o:e:l:d:-:n:f:c")) != EOF) {
+   while ((ch = getopt(args,argv,"VvuhqU:r:o:e:l:d:-:n:f:ci")) != EOF) {         
       switch(ch) {
        case 'h':
          usage();
@@ -1200,6 +1257,9 @@ main(int args, char **argv)
          }
          max_queries = atoi(optarg);
          break;
+       case 'i':
+         NOCASE=1;         
+         break;
        default:
          return(1);
          break;
@@ -1218,8 +1278,14 @@ main(int args, char **argv)
       int i = 0;
       
       while (j < args) {
-         while ((database = SLOCATE_PATH[i++])) res |= decode_db(database, argv[j++]);
-         i = 0;
+         /* while ((database = SLOCATE_PATH[i++])) res |= decode_db(database, argv[j++]);
+            i = 0;
+          * A Bug fix by Hans-Juergen Godau <godau@wi-inf.uni-essen.de>
+          * Prevents segfault when using multiple databases */
+         while ((database = SLOCATE_PATH[i++]))
+           res |= decode_db(database, argv[j]);
+         i = 0; j += 1;
+         
       }
    } else
        usage();
